{******************************************************************************}
{                                                                              }
{   Copyright (c) 2002 Borland Software Corporation                            }
{   Contributions by Dmitry Streblechenko (dmitry@dimastr.com)                 }
{                                                                              }
{******************************************************************************}

unit OfficeAddins;

interface

uses Windows, Classes, ComObj, OleCtrls, ActiveX, OleServer, Registry;

const

  AddinLoadDisconnected     = 0;
  AddinLoadConnected        = 1;
  AddinLoadBootLoad         = 2;
  AddinLoadDemandLoad       = 8;
  AddinLoadConnectFirstTime = 16;

type
  TOfficeVersion = (ov2000, ov2002);
  TOfficeAppKind = (oaAccess, oaExcel, oaFrontPage, oaOutlook, oaPowerPoint, oaWord);

type
  ext_ConnectMode = TOleEnum;
const
  ext_cm_AfterStartup = $00000000;
  ext_cm_Startup      = $00000001;
  ext_cm_External     = $00000002;
  ext_cm_CommandLine  = $00000003;

// Constants for enum ext_DisconnectMode
type
  ext_DisconnectMode = TOleEnum;
const
  ext_dm_HostShutdown = $00000000;
  ext_dm_UserClosed   = $00000001;

type

  IDTExtensibility2 = interface(IDispatch)
    ['{B65AD801-ABAF-11D0-BB8B-00A0C90F2744}']
    procedure OnConnection(const Application: IDispatch; ConnectMode: ext_ConnectMode;
                           const AddInInst: IDispatch; var custom: PSafeArray); safecall;
    procedure OnDisconnection(RemoveMode: ext_DisconnectMode; var custom: PSafeArray); safecall;
    procedure OnAddInsUpdate(var custom: PSafeArray); safecall;
    procedure OnStartupComplete(var custom: PSafeArray); safecall;
    procedure OnBeginShutdown(var custom: PSafeArray); safecall;
  end;

  TOfficeAddinFactory = class(TComObjectFactory)
  private
    FFriendlyName : string;
    FDescription  : string;
    FLoadBehavior : integer;
    FOfficeApp    : TOfficeAppKind;
    FRootKey      : HKEY;
  public
    constructor Create(ComServer: TComServerObject;
                       ComClass: TComClass;
                       const ClassID: TGUID;
                       FriendlyName : string;
                       ClassName : string;
                       Description : string;
                       LoadBevavior : integer;
                       OfficeApp : TOfficeAppKind;
                       RootKey : HKEY);
    procedure UpdateRegistry(Register: Boolean); override;
  end;

implementation

{ TOfficeAddinFactory }

constructor TOfficeAddinFactory.Create(ComServer: TComServerObject;
  ComClass: TComClass; const ClassID: TGUID; FriendlyName: string;
  ClassName : string; Description : string;
  LoadBevavior: integer; OfficeApp : TOfficeAppKind;
  RootKey : HKEY);
begin
  FFriendlyName := FriendlyName;
  FDescription  := Description;
  FLoadBehavior := LoadBevavior;
  FOfficeApp    := OfficeApp;
  FRootKey      := RootKey;
  inherited Create(ComServer, ComClass, ClassID, ClassName, FriendlyName,
                   ciMultiInstance, tmSingle);
end;

procedure TOfficeAddinFactory.UpdateRegistry(Register: Boolean);
var
  Reg: TRegistry;
  strAppName: string;
begin
  inherited;
  case FOfficeApp of
    oaAccess     : strAppName := 'Access';
    oaExcel      : strAppName := 'Excel';
    oaFrontPage  : strAppName := 'FrontPage';
    oaOutlook    : strAppName := 'Outlook';
    oaPowerPoint : strAppName := 'PowerPoint';
    oaWord       : strAppName := 'Word';
  end;
  Reg:=TRegistry.Create;
  try
    Reg.RootKey:=FRootKey;
    if Register then begin
      if Reg.OpenKey('Software\Microsoft\Office\'+strAppName+'\Addins\'+GetProgID, TRUE) then begin
        Reg.WriteString('FriendlyName', FFriendlyName);
        Reg.WriteString('Description', FDescription);
        Reg.WriteInteger('LoadBehavior', FLoadBehavior);
      end;
    end
    else begin
      if Reg.KeyExists('Software\Microsoft\Office\'+strAppName+'\Addins\'+GetProgID) then begin
        Reg.DeleteKey('Software\Microsoft\Office\'+strAppName+'\Addins\'+GetProgID);
      end;
    end;
  finally
    Reg.Free;
  end;
end;

end.
